@@ -28,19 +28,15 @@
 #endif
 #include "fribidi_unicode.h"
 
-#ifndef SIZEOF_CHAR
-#define SIZEOF_CHAR sizeof (char)
-#endif
-
 static void
-err (char *msg)
+err (const char *msg)
 {
   fprintf (stderr, "fribidi_create_char_types: %s\n", msg);
   exit (1);
 }
 
 static void
-err2 (char *fmt, char *p)
+err2 (const char *fmt, const char *p)
 {
   fprintf (stderr, "fribidi_create_char_types: ");
   fprintf (stderr, fmt, p);
@@ -51,7 +47,7 @@ err2 (char *fmt, char *p)
 /* *INDENT-OFF* */
 struct
 {
-  char *name;
+  const char *name;
   int key;
 }
 type_names[] =
@@ -85,7 +81,7 @@ static char *names[type_names_count];
 static char *unidata_file;
 
 static char
-get_type (char *s)
+get_type (const char *s)
 {
   int i;
 
@@ -104,15 +100,17 @@ get_type (char *s)
 #define char_name "FriBidiChar"
 #define prop_to_type_name "fribidi_prop_to_type"
 #define default_type "LTR"
+#define export_api "FRIBIDI_API"
 
 static int table[FRIBIDI_UNICODE_CHARS];
 
 static void
-init_table ()
+init_table (void)
 {
   int i;
-  int deftype = get_type (default_type), RTL = get_type ("RTL"), AL =
-    get_type ("AL");
+  register FriBidiChar c;
+  int deftype = get_type (default_type),
+    RTL = get_type ("RTL"), AL = get_type ("AL"), BN = get_type ("BN");
 
   for (i = 0; i < type_names_count; i++)
     names[i] = 0;
@@ -120,22 +118,46 @@ init_table ()
     names[type_names[i].key] = type_names[i].name;
 
   /* initialize table */
-  for (i = 0; i < FRIBIDI_UNICODE_CHARS; i++)
-    table[i] = deftype;
-  for (i = 0x590; i < 0x600; i++)
-    table[i] = RTL;
-  for (i = 0xFB1D; i < 0xFB50; i++)
-    table[i] = RTL;
-  for (i = 0x600; i < 0x7C0; i++)
-    table[i] = AL;
-  for (i = 0xFB50; i < 0xFE00; i++)
-    table[i] = AL;
-  for (i = 0xFE70; i < 0xFF00; i++)
-    table[i] = AL;
+  for (c = 0; c < FRIBIDI_UNICODE_CHARS; c++)
+    table[c] = deftype;
+
+  for (c = 0x0590; c < 0x0600; c++)
+    table[c] = RTL;
+  for (c = 0x07C0; c < 0x0900; c++)
+    table[c] = RTL;
+  for (c = 0xFB1D; c < 0xFB50; c++)
+    table[c] = RTL;
+
+  for (c = 0x0600; c < 0x07C0; c++)
+    table[c] = AL;
+  for (c = 0xFB50; c < 0xFDD0; c++)
+    table[c] = AL;
+  for (c = 0xFDF0; c < 0xFE00; c++)
+    table[c] = AL;
+  for (c = 0xFE70; c < 0xFF00; c++)
+    table[c] = AL;
+
+  for (c = 0x2060; c < 0x2070; c++)
+    table[c] = BN;
+  for (c = 0xFDD0; c < 0xFDF0; c++)
+    table[c] = BN;
+  for (c = 0xFFF0; c < 0xFFF9; c++)
+    table[c] = BN;
+  for (c = 0xFFFF; c < FRIBIDI_UNICODE_CHARS; c += 0x10000)
+    table[c - 1] = table[c] = BN;
+
+  if (FRIBIDI_UNICODE_CHARS > 0x10000)
+    {
+      for (c = 0x10800; c < 0x11000; c++)
+	table[c] = RTL;
+
+      for (c = 0xE0000; c < 0xE1000; c++)
+	table[c] = BN;
+    }
 }
 
 static void
-read_unicode_data ()
+read_unicode_data (void)
 {
   char s[500], tp[10];
   unsigned int i;
@@ -153,7 +175,7 @@ read_unicode_data ()
 }
 
 static char *
-headermacro (char *file)
+headermacro (const char *file)
 {
   char *t = strdup (file);
   char *p = t;
@@ -169,7 +191,7 @@ headermacro (char *file)
 }
 
 static void
-write_char_type (char *file, int max_depth)
+write_char_type (const char *file, int max_depth)
 {
   int i;
   FILE *f;
@@ -190,8 +212,12 @@ write_char_type (char *file, int max_depth)
       fprintf (f, "#define %s FRIBIDI_PROP_TYPE_%s\n", names[i], names[i]);
   fprintf (f, "\n");
 
+  fprintf (f, "#define PACKTAB_UINT8 fribidi_uint8\n");
+  fprintf (f, "#define PACKTAB_UINT16 fribidi_uint16\n");
+  fprintf (f, "#define PACKTAB_UINT32 fribidi_uint32\n");
+
   if (!pack_table
-      (table, FRIBIDI_UNICODE_CHARS, SIZEOF_CHAR, max_depth, 3, names,
+      (table, FRIBIDI_UNICODE_CHARS, sizeof (char), max_depth, 3, names,
        key_type_name, table_name, macro_name, f))
     err ("error: insufficient memory, decrease max_depth");
 
@@ -203,7 +229,7 @@ write_char_type (char *file, int max_depth)
 	   "/*======================================================================\n"
 	   " *  %s() returns the bidi type of a character.\n"
 	   " *----------------------------------------------------------------------*/\n"
-	   "%s\n"
+	   "%s %s\n"
 	   "%s (%s uch)\n"
 	   "{\n"
 	   "  if (uch < 0x%x)\n"
@@ -213,8 +239,8 @@ write_char_type (char *file, int max_depth)
 	   "  /* Non-Unicode chars */\n"
 	   "}\n"
 	   "\n",
-	   function_name, char_type_name, function_name, char_name,
-	   FRIBIDI_UNICODE_CHARS, prop_to_type_name, macro_name,
+	   function_name, export_api, char_type_name, function_name,
+	   char_name, FRIBIDI_UNICODE_CHARS, prop_to_type_name, macro_name,
 	   default_type);
   fprintf (f, "\n#endif /* %s */\n", FILENAME);
 
